import numpy as np;
import openturns as ot;

def bretschneider(F, u10, g, d):
    '''
    Compute wave characteristics from wind and water depth.
    
    Bretschneider equation (TAW 1989 Bretschneijder)
    
    Arguments:
    F: wind fetch [m]
    u10: wind speed [m/s]
    g: gravity [m/s^2]
    d: water depth [m]
    
    Returns:
    Hm0: wave height [m]
    Tm: wave period [s]
    '''
    Fdak = (F * g) / (u10 ** 2)
    ddak = (d * g) / (u10 ** 2)
    Hdak = 0.283 * np.tanh(0.53 * ddak**0.75) * np.tanh((0.0125 * Fdak**0.42) / np.tanh(0.53 * ddak**0.75))
    Tdak = 2.4 * np.pi * np.tanh(0.833 * ddak**0.375) * np.tanh((0.077 * Fdak**0.25) / np.tanh(0.833 * ddak**0.375))
    T13 = (Tdak * u10) / g
    Tp = 1.08 * T13
    Hm0 = (Hdak * u10**2) / g
    Tm = Tp / 1.1
    return Hm0, Tm

def overtopping_discharge(bottom, h, u10):
    '''
    Calculate overtopping discharge from wave and freeboard
    
    An empirical equation that is set up here to use the wave conditions from
    bretschneider() function. Many variables are hard-coded in this function
    for use in the HOS prob design unit exercise. Returns discharge as a
    volumetric flow per unit length of dike.
    
    EurOtop equation 5.10 & 5.11.
    
    Arguments:
    bottom: elevation of water body bottom [m + NAP]
    h: elevation of water body surface [m + NAP]
    u10: wind speed [m/s]
    
    Returns:
    qload: overtopping discharge [L/m/s]
    '''
    g = 9.81  # m/s^2
    toe = 1  # m
    crest = 7  # m
    tanalpha = 1/5
    yf = 1
    Ab = ot.Normal(0.023,0.003)
    Ab = Ab.computeQuantile(0.95)[0]
    Bb = ot.Normal(2.7,0.2)
    Bb = Bb.computeQuantile(0.05)[0]
    Anb = ot.Normal(0.09,0.0135)
    Anb = Anb.computeQuantile(0.95)[0]
    Bnb = ot.Normal(1.5,0.15)
    Bnb = Bnb.computeQuantile(0.05)[0]
    yb = 1
    yv = 1
    F = 34060  # m
    beta = 46  # degree
    kappa = 3.2e-06
    ms = 3
    WindSetup = 1
    
    # Water depth near toe
    d = h - toe;

    # Water depth away from toe (lake, sea, river, etc)
    db = h - bottom;

    # Make sure that the water depth at the toe cannot be negative, otherwise, the waveheight becomes a complex number
    if(isinstance(d, np.ndarray)):
        d[d <= 0] = 0.000001;
    else:
        if(d <= 0):
            d = 0.000001;

    # Make sure that the water depth in the lake, sea, etc. cannot be negative, otherwise, the waveheight becomes a complex number
    if(isinstance(db, np.ndarray)):
        db[db <= 0] = 0.000001;
    else:
        if(db <= 0):
            db = 0.000001;

    # Wind setup
    if(WindSetup == True):
        setup = ms * 0.5 * kappa * u10**2 * F * np.cos((beta / 360) * 2 * np.pi) / (g * db)
        h = h + setup;

    # u10 cannot be negative
    if(isinstance(u10, np.ndarray)):
        u10[u10 <= 0] = 0.0001;
    else:     
        if(u10 <= 0):
            u10 = 0.0001;

    # Fetch cannot be 0
    if(isinstance(F, np.ndarray)):
        F[F <= 0] = 0.0001;
    else:     
        if(F <= 0):
            F = 0.0001;

    # Calculate the freeboard and water depth at the toe
    Rc = crest - h

    # Retrieve all required wave data (TAW 1989 Bretschneijder)
    Hm0, Tm = bretschneider(F,u10,g,db);

    # Incident waves
    if(isinstance(beta, np.ndarray)):
        ybeta = np.ones(len(beta));
        for i in range(len(beta)):
            if(beta[i] <= 80):
                ybeta[i] = 1 - 0.0033 * np.abs(beta[i]);
            elif(beta[i] <= 110):
                ybeta[i] = 1 - 0.0033 * np.abs(80);
                Tm[i] = Tm[i] * ((110 - np.abs(beta[i])) / 30) ** 0.5;
                Hm0[i] = Hm0[i] * (110 - np.abs(beta[i])) / 30;
    else:
        ybeta = 1;
        if(beta <= 80):
            ybeta = 1 - 0.0033 * np.abs(beta);
        elif(beta <= 110):
            ybeta = 1 - 0.0033 * np.abs(80);
            Tm = Tm * ((110 - np.abs(beta)) / 30) ** 0.5;
            Hm0 = Hm0 * (110 - np.abs(beta)) / 30;

    # Calculate the iribarren number
    L0 = (g * Tm**2) / (2 * np.pi)
    ib = tanalpha / np.sqrt(Hm0 / L0)

    # EurOtop page 122, for a negative freeboard, you should use Rc = 0 for overtopping
    if(isinstance(Rc, np.ndarray)):
        Rc[Rc < 0] = 0;
    else:
        if(Rc < 0):
            Rc = 0;

    # Calculate the overtopping discharge (EurOtop equation 5.10)
    overtopping = (Ab / np.sqrt(tanalpha)) * yb * ib * np.exp(-((Bb * Rc) / (yb * yf * ybeta * yv * ib * Hm0))**1.3) * np.sqrt(g * Hm0**3)

    # Check if the calculated discharge is not larger than given by EurOtop equation 5.11 (non breaking waves)
    qnb = Anb * np.exp(-((Bnb * Rc) / (yb * yf * ybeta * yv * Hm0))**1.3) * np.sqrt(g * Hm0**3)
    if(isinstance(overtopping, np.ndarray)):
        for i in range(len(overtopping)):
            if(overtopping[i] > qnb[i]):
                overtopping[i] = qnb[i];
    else:
        if(overtopping > qnb):
            overtopping = qnb;

    if(isinstance(beta, np.ndarray)):
        overtopping[beta > 110] = 0;
    else:
        if(beta > 110):
            overtopping = 0;

    # Construct the total q from the overtopping part, and if the freeboard is negative, the overflow part. Multiply by 1000 to retrieve l/m/s
    Rc = crest - h;
    qload = (overtopping + np.heaviside(-Rc,1) * 0.54 * np.sqrt(g * np.absolute(Rc**3))) * 1000

    return qload